@@ -21,23 +21,29 @@
 //! ```no_run
 //! use manic::Downloader;
 //! #[tokio::main]
-//! async fn main() -> Result<(), manic::Error> {
+//! async fn main() -> Result<(), manic::ManicError> {
 //!     let number_of_concurrent_tasks: u8 = 5;
 //!     let client = Downloader::new("https://crates.io", number_of_concurrent_tasks).await?;
 //!     let result = client.download().await?;
 //!     Ok(())
 //! }
 //! ```
-
+#[macro_use]
+extern crate derive_builder;
 mod chunk;
 mod downloader;
 mod error;
 mod hash;
+mod multi;
 
 pub use downloader::Downloader;
-pub use error::{Error, Result};
+pub use downloader::DownloaderBuilder;
+pub use error::{ManicError, Result};
 pub use hash::Hash;
 #[cfg(feature = "progress")]
 pub use indicatif::ProgressStyle;
-pub use reqwest::Client;
-pub use reqwest::Url;
+pub use multi::Downloaded;
+pub use multi::Map;
+pub use multi::MultiDownloader;
+pub use multi::MultiDownloaderBuilder;
+pub use reqwest::{header, Client, Url};
